/******************************************************************************
  Copyright (c) 1992 Xerox Corporation.  All rights reserved.
  Portions of this code were written by Stephen White, aka ghond.
  Use and copying of this software and preparation of derivative works based
  upon this software are permitted.  Any distribution of this software or
  derivative works must comply with all applicable United States export
  control laws.  This software is made available AS IS, and Xerox Corporation
  makes no warranty about the software, its performance or its conformity to
  any specification.  Any person obtaining a copy of this software is requested
  to send their name and post office or electronic mail address to:
    Pavel Curtis
    Xerox PARC
    3333 Coyote Hill Rd.
    Palo Alto, CA 94304
    Pavel@Xerox.Com
 *****************************************************************************/

#include "db.h"
#include "functions.h"
#include "utils.h"

static package
bf_properties(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object) */
    Var r;

    r = get_property_list(arglist.v.list[1].v.obj, progr);

    free_var(arglist);
    if (r.type == TYPE_ERR)
	return make_error_pack(r.v.err);
    else
	return make_var_pack(r);
}


static package
bf_prop_info(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object, prop-name) */
    Var r;

    r = get_prop_info(arglist.v.list[1].v.obj, arglist.v.list[2].v.str, progr);
    free_var(arglist);
    
    if (r.type == TYPE_ERR)
	return make_error_pack(r.v.err);
    else
	return make_var_pack(r);
}

static package
bf_set_prop_info(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object, prop-name, {owner, perms}) */
    enum error e = set_prop_info(arglist.v.list[1].v.obj,
				 arglist.v.list[2].v.str, progr,
				 arglist.v.list[3]);
    free_var(arglist);
    if (e != E_NONE)
	return make_error_pack(e);
    else
	return no_var_pack();
}

static package
bf_add_prop(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object, prop-name, initial-value, initial-info) */
    enum error  e = add_property(arglist.v.list[1].v.obj,
				 arglist.v.list[2].v.str, progr,
				 arglist.v.list[3], arglist.v.list[4]);
    free_var(arglist);
    if (e != E_NONE) 
	return make_error_pack(e);
    else
	return no_var_pack();
}

static package
bf_delete_prop(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object, prop-name) */

    enum error e = delete_property(arglist.v.list[1].v.obj,
				   arglist.v.list[2].v.str, progr);
    free_var(arglist);
    if (e != E_NONE)
	return make_error_pack(e);
    else
	return no_var_pack();
}

static package
bf_clear_prop(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object, prop-name) */
    Var value;
    enum error e;

    value.type = TYPE_CLEAR;
    e = set_prop_val(arglist.v.list[1].v.obj,
		     arglist.v.list[2].v.str, value, progr);
    free_var(arglist);
    if (e != E_NONE) 
	return make_error_pack(e);
    else
	return no_var_pack();
}

static package
bf_is_clear_prop(Var arglist, Byte next, void *vdata, Objid progr)
{ /* (object, prop-name) */
    Var r;
    Var value;
    enum error e;

    e = get_prop_val(arglist.v.list[1].v.obj,
		     arglist.v.list[2].v.str,
		     &value, progr, 1);
    free_var(arglist);
    if (e != E_NONE) 
	return make_error_pack(e);
    
    free_var(value);		/* the .type field is still good for below */
    r.type = TYPE_NUM;
    if (value.type == TYPE_CLEAR)
	r.v.num = 1;
    else
	r.v.num = 0;
    return make_var_pack(r);
}

void
register_property(void)
{ 
    (void) register_function("properties", 1, 1, bf_properties, TYPE_OBJ);
    (void) register_function("property_info", 2, 2, bf_prop_info, TYPE_OBJ, TYPE_STR);
    (void) register_function("set_property_info", 3, 3, bf_set_prop_info,
			     TYPE_OBJ, TYPE_STR, TYPE_LIST);
    (void) register_function("add_property", 4, 4, bf_add_prop, TYPE_OBJ, TYPE_STR,
			     TYPE_ANY, TYPE_LIST);
    (void) register_function("delete_property", 2, 2, bf_delete_prop,
			     TYPE_OBJ, TYPE_STR);
    (void) register_function("clear_property", 2, 2, bf_clear_prop,
			     TYPE_OBJ, TYPE_STR);
    (void) register_function("is_clear_property", 2, 2, bf_is_clear_prop,
			     TYPE_OBJ, TYPE_STR);
}

char rcsid_property[] = "$Id: property.c,v 1.12 1992/10/23 23:03:47 pavel Exp $";

/* $Log: property.c,v $
 * Revision 1.12  1992/10/23  23:03:47  pavel
 * Added copyright notice.
 *
 * Revision 1.11  1992/09/14  18:39:04  pjames
 * Updated #includes.  Moved rcsid to bottom.
 *
 * Revision 1.10  1992/09/14  17:42:25  pjames
 * Moved db_modification code to db modules.
 *
 * Revision 1.9  1992/09/08  22:05:03  pjames
 * changed `register_bf_prop()' to `register_property()'
 *
 * Revision 1.8  1992/09/04  22:41:36  pavel
 * Fixed some picky ANSI C problems with (const char *)'s.
 *
 * Revision 1.7  1992/09/04  01:16:50  pavel
 * Added support for the `f' (for `fertile') bit on objects.
 *
 * Revision 1.6  1992/09/03  16:28:46  pjames
 * Added `new_propdef()' and `str_hash()' for hash-lookup of properties.
 * Changed code to handle new representation of property definitions (arrays).
 * Added TYPE_CLEAR handling.
 * Added built-in functions `clear_property()' and `is_clear_property()'.
 *
 * Revision 1.5  1992/08/31  22:25:39  pjames
 * Changed some `char *'s to `const char *'
 *
 * Revision 1.4  1992/08/28  16:18:45  pjames
 * Changed myfree(*, M_STRING) to free_str(*).
 * Changed some strref's to strdup.
 * Changed vardup to varref.
 *
 * Revision 1.3  1992/08/10  17:41:48  pjames
 * Moved get_bi_prop, get_prop, set_bi_prop, and put_prop from execute.c.
 *
 * Revision 1.2  1992/07/21  00:06:01  pavel
 * Added rcsid_<filename-root> declaration to hold the RCS ident. string.
 *
 * Revision 1.1  1992/07/20  23:23:12  pavel
 * Initial RCS-controlled version.
 */
